import androidx.compose.animation.EnterTransition
import androidx.compose.animation.ExitTransition
import androidx.compose.animation.core.tween
import androidx.compose.animation.slideIn
import androidx.compose.animation.slideOut
import androidx.compose.runtime.Composable
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.IntOffset
import androidx.navigation.NavGraph.Companion.findStartDestination
import androidx.navigation.NavHostController
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.toRoute
import fr.openium.consentium.R
import fr.openium.consentium.api.Consentium
import fr.openium.consentium.ui.screens.main.MainScreen
import fr.openium.consentium.ui.screens.splash.SplashScreen
import fr.openium.consentium.ui.theme.Error
import fr.openium.consentium.ui.theme.OnPrimary
import fr.openium.consentium.ui.theme.OnSecondary
import fr.openium.consentium.ui.theme.OnSurface
import fr.openium.consentium.ui.theme.OnSurfaceVariant
import fr.openium.consentium.ui.theme.Primary
import fr.openium.consentium.ui.theme.Secondary
import fr.openium.consentium.ui.theme.Success
import fr.openium.consentium.ui.theme.SurfaceHigh
import fr.openium.consentium.ui.theme.SurfaceHighest
import fr.openium.consentium.ui.theme.SurfaceMiddle
import fr.openium.consentium.ui.theme.Tertiary
import fr.openium.consentium_ui.ui.components.ConsentiumComponent
import fr.openium.consentium_ui.ui.components.style.ConsentiumDefaults
import fr.openium.consentium_ui.ui.model.ConsentiumPageUI

private const val NAV_ANIMATION_TIME = 500

private val SLIDE_IN_FROM_RIGHT_ENTER_TRANSITION = slideIn(
    animationSpec = tween(NAV_ANIMATION_TIME),
    initialOffset = { fullSize -> IntOffset(x = fullSize.height, y = 0) }
)

private val SLIDE_IN_FROM_LEFT_ENTER_TRANSITION = slideIn(
    animationSpec = tween(NAV_ANIMATION_TIME),
    initialOffset = { fullSize -> IntOffset(x = -fullSize.height, y = 0) }
)

private val SLIDE_OUT_TO_LEFT_EXIT_TRANSITION = slideOut(
    animationSpec = tween(NAV_ANIMATION_TIME),
    targetOffset = { fullSize -> IntOffset(x = -fullSize.height, y = 0) }
)

private val SLIDE_OUT_TO_RIGHT_EXIT_TRANSITION = slideOut(
    animationSpec = tween(NAV_ANIMATION_TIME),
    targetOffset = { fullSize -> IntOffset(x = fullSize.height, y = 0) }
)

private infix fun String?.ifIn(destinations: List<Destination>): Boolean {
    return this in destinations.map { it::class.qualifiedName }
}

private infix fun Boolean.thenTransitionWith(transition: EnterTransition): EnterTransition? {
    return if (this) transition else null
}

private infix fun EnterTransition?.elseTransitionWith(transition: EnterTransition): EnterTransition {
    return this ?: transition
}

private infix fun Boolean.thenTransitionWith(transition: ExitTransition): ExitTransition? {
    return if (this) transition else null
}

private infix fun ExitTransition?.elseTransitionWith(transition: ExitTransition): ExitTransition {
    return this ?: transition
}

@Composable
fun DemoNavGraph(navHostController: NavHostController) {

    NavHost(
        navController = navHostController,
        startDestination = Destination.Splash,
    ) {

        composable<Destination.Splash>(
            enterTransition = {
                SLIDE_IN_FROM_RIGHT_ENTER_TRANSITION
            },
            exitTransition = {
                SLIDE_OUT_TO_LEFT_EXIT_TRANSITION
            }
        ) {
            SplashScreen(
                navigateToMain = {
                    navHostController.navigate(Destination.Main) {
                        popUpTo(navHostController.graph.findStartDestination().id) {
                            saveState = true
                        }
                    }
                },
                navigateToConsent = {
                    navHostController.navigate(Destination.Consent(ConsentiumPageUI.GENERAL_CONSENT))
                }
            )
        }

        composable<Destination.Main>(
            enterTransition = {
                initialState.destination.route ifIn listOf(
                    Destination.Splash
                ) thenTransitionWith SLIDE_IN_FROM_RIGHT_ENTER_TRANSITION elseTransitionWith SLIDE_IN_FROM_LEFT_ENTER_TRANSITION
            },
            exitTransition = {
                targetState.destination.route ifIn listOf(
                    Destination.Splash
                ) thenTransitionWith SLIDE_OUT_TO_RIGHT_EXIT_TRANSITION elseTransitionWith SLIDE_OUT_TO_LEFT_EXIT_TRANSITION
            }
        ) {
            MainScreen(
                onGoToConsentMaster = {
                    navHostController.navigate(Destination.Consent(ConsentiumPageUI.GENERAL_CONSENT))
                },
                onGoToConsentDetail = {
                    navHostController.navigate(Destination.Consent(ConsentiumPageUI.DETAILS_CONSENT))
                },
            )
        }

        composable<Destination.Consent>(
            enterTransition = {
                initialState.destination.route ifIn listOf(
                    Destination.Splash,
                    Destination.Main,
                ) thenTransitionWith SLIDE_IN_FROM_RIGHT_ENTER_TRANSITION elseTransitionWith SLIDE_IN_FROM_LEFT_ENTER_TRANSITION
            },
            exitTransition = {
                targetState.destination.route ifIn listOf(
                    Destination.Splash,
                    Destination.Main,
                ) thenTransitionWith SLIDE_OUT_TO_RIGHT_EXIT_TRANSITION elseTransitionWith SLIDE_OUT_TO_LEFT_EXIT_TRANSITION
            }
        ) { backStackEntry ->
            val consent = backStackEntry.toRoute<Destination.Consent>()

            val appId = stringResource(R.string.app_id)
            val apiKey = stringResource(R.string.api_key)

            val context = LocalContext.current
            ConsentiumComponent(
                defaultLandingPage = consent.landingPage,
                onQuitConsent = {
                    navHostController.navigate(Destination.Main) {
                        popUpTo(navHostController.graph.findStartDestination().id) {
                            saveState = true
                        }
                    }
                },
                consentium = Consentium(
                    context = context,
                    apiKey = apiKey,
                    appId = appId,
                ),
                colors = ConsentiumDefaults.colors(
                    primary = Primary,
                    onPrimary = OnPrimary,
                    secondary = Secondary,
                    onSecondary = OnSecondary,
                    tertiary = Tertiary,
                    onSurfaceVariant = OnSurfaceVariant,
                    onSurface = OnSurface,
                    error = Error,
                    surfaceHighest = SurfaceHighest,
                    surfaceHigh = SurfaceHigh,
                    surfaceMiddle = SurfaceMiddle,
                    success = Success,
                )
            )
        }
    }
}
